#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
.. module:: image_conversion
   :platform: Unix, Windows
   :synopsis: A module to handle image conversion for falconz.

.. moduleauthor:: Lalith Kumar Shiyam Sundar

This module handles image conversion for the falconz.

Usage:
    The functions in this module can be imported and used in other modules within the falconz to perform image conversion.

"""
import SimpleITK
import contextlib
import dcm2niix
import dicom2nifti
import io
import logging
import nibabel as nib
import os
import pydicom
import re
import unicodedata
from typing import Optional

from falconz import file_utilities as fop


class ImageConverter:
    def __init__(self, input_directory: str, output_directory: str):
        """
        Initializes an ImageConverter object.

        :param input_directory: The path to the input directory.
        :type input_directory: str
        :param output_directory: The path to the output directory.
        :type output_directory: str
        """
        self.input_directory = input_directory
        self.output_directory = output_directory
        self.split_nifti_directory = os.path.join(self.output_directory, 'split-nifti-files')

        if not os.path.exists(self.split_nifti_directory):
            os.makedirs(self.split_nifti_directory)

    def non_nifti_to_nifti(self, input_path: str, output_directory: Optional[str] = None) -> None:
        """
        Converts non-NIfTI files to NIfTI format.

        :param input_path: The path to the input file or directory.
        :type input_path: str
        :param output_directory: The path to the output directory.
        :type output_directory: str
        :return: None
        """
        if not os.path.exists(input_path):
            print(f"Input path {input_path} does not exist.")
            return

        if os.path.isdir(input_path):
            dicom_info = self.create_dicom_lookup(input_path)
            nifti_dir = self.dcm2niix(input_path, output_directory)
            self.rename_nifti_files(nifti_dir, dicom_info)
            return

        if os.path.isfile(input_path):
            _, filename = os.path.split(input_path)
            if filename.startswith('.') or filename.endswith(('.nii.gz', '.nii')):
                return
            else:
                output_image = SimpleITK.ReadImage(input_path)
                output_image_basename = f"{os.path.splitext(filename)[0]}.nii"

        if output_directory is None:
            output_directory = os.path.dirname(input_path)

        output_image_path = os.path.join(output_directory, output_image_basename)
        SimpleITK.WriteImage(output_image, output_image_path)

    def dcm2niix(self, input_path: str, output_dir: str) -> str:
        """
        Converts DICOM files to NIfTI format using dcm2niix.

        :param input_path: The path to the input directory.
        :type input_path: str
        :param output_dir: The path to the output directory.
        :type output_dir: str
        :return: The path to the output directory.
        :rtype: str
        """
        # Save the original stdout and stderr file descriptors.
        original_stdout_fd = os.dup(1)
        original_stderr_fd = os.dup(2)

        # Open a null file to redirect the output.
        with open(os.devnull, 'w') as null_file:
            null_fd = null_file.fileno()

            # Redirect stdout and stderr to the null file.
            os.dup2(null_fd, 1)
            os.dup2(null_fd, 2)

            try:
                dcm2niix.main(['-z', 'y', '-o', output_dir, input_path])
            finally:
                # Restore the original stdout and stderr file descriptors.
                os.dup2(original_stdout_fd, 1)
                os.dup2(original_stderr_fd, 2)
                os.close(original_stdout_fd)
                os.close(original_stderr_fd)

        return output_dir

    def remove_accents(self, unicode_filename):
        """
        Removes accents from a Unicode filename.

        :param unicode_filename: The Unicode filename to remove accents from.
        :type unicode_filename: str
        :return: The filename without accents.
        :rtype: str
        """
        try:
            unicode_filename = str(unicode_filename).replace(" ", "_")
            cleaned_filename = unicodedata.normalize('NFKD', unicode_filename).encode('ASCII', 'ignore').decode('ASCII')
            cleaned_filename = re.sub(r'[^\w\s-]', '', cleaned_filename.strip().lower())
            cleaned_filename = re.sub(r'[-\s]+', '-', cleaned_filename)
            return cleaned_filename
        except:
            return unicode_filename

    def is_dicom_file(self, filename):
        """
        Determines if a file is a DICOM file.

        :param filename: The name of the file to check.
        :type filename: str
        :return: True if the file is a DICOM file, False otherwise.
        :rtype: bool
        """
        try:
            pydicom.dcmread(filename)
            return True
        except pydicom.errors.InvalidDicomError:
            return False

    def create_dicom_lookup(self, dicom_dir):
        """
        Creates a dictionary of DICOM file information.

        :param dicom_dir: The path to the directory containing DICOM files.
        :type dicom_dir: str
        :return: A dictionary of DICOM file information.
        :rtype: dict
        """
        dicom_info = {}
        for filename in os.listdir(dicom_dir):
            full_path = os.path.join(dicom_dir, filename)
            if self.is_dicom_file(full_path):
                ds = pydicom.dcmread(full_path)
                series_number = ds.SeriesNumber if 'SeriesNumber' in ds else None
                series_description = ds.SeriesDescription if 'SeriesDescription' in ds else None
                sequence_name = ds.SequenceName if 'SequenceName' in ds else None
                protocol_name = ds.ProtocolName if 'ProtocolName' in ds else None
                series_instance_UID = ds.SeriesInstanceUID if 'SeriesInstanceUID' in ds else None
                if ds.Modality == 'PT':
                    modality = 'PET'
                else:
                    modality = ds.Modality

                if series_number is not None:
                    base_filename = self.remove_accents(series_number)
                    if series_description is not None:
                        anticipated_filename = f"{base_filename}_{self.remove_accents(series_description)}.nii"
                    elif sequence_name is not None:
                        anticipated_filename = f"{base_filename}_{self.remove_accents(sequence_name)}.nii"
                    elif protocol_name is not None:
                        anticipated_filename = f"{base_filename}_{self.remove_accents(protocol_name)}.nii"
                else:
                    anticipated_filename = f"{self.remove_accents(series_instance_UID)}.nii"

                dicom_info[anticipated_filename] = modality
        return dicom_info

    def rename_nifti_files(self, nifti_dir, dicom_info):
        """
        Renames NIfTI files based on DICOM file information.

        :param nifti_dir: The path to the directory containing NIfTI files.
        :type nifti_dir: str
        :param dicom_info: A dictionary of DICOM file information.
        :type dicom_info: dict
        :return: None
        """
        for filename in os.listdir(nifti_dir):
            if filename.endswith('.nii'):
                modality = dicom_info.get(filename, '')
                if modality:
                    new_filename = f"{modality}_{filename}"
                    os.rename(os.path.join(nifti_dir, filename), os.path.join(nifti_dir, new_filename))
                    del dicom_info[filename]

    def split4d(self, nifti_file: str, out_dir: str) -> None:
        """
        Splits a 4D NIfTI file into 3D NIfTI files.

        :param nifti_file: The path to the 4D NIfTI file.
        :type nifti_file: str
        :param out_dir: The path to the output directory.
        :type out_dir: str
        :return: None
        """
        split_nifti_files = nib.funcs.four_to_three(nib.funcs.squeeze_image(nib.load(nifti_file)))
        i = 0
        for file in split_nifti_files:
            # vol should be named as vol0000.nii, vol0001.nii, etc.

            nib.save(file, os.path.join(out_dir, 'vol' + str(i).zfill(4) + '.nii.gz'))
            i += 1

    def convert(self):
        """
        Converts input files to NIfTI format and splits 4D NIfTI files into 3D NIfTI files.

        :return: The path to the directory containing the split NIfTI files.
        :rtype: str
        """
        if os.path.isdir(self.input_directory):
            self.non_nifti_to_nifti(self.input_directory, self.output_directory)
        else:
            print(f"Input {self.input_directory} is not a directory.")
            return

        nifti_files = []
        for root, dirs, files in os.walk(self.output_directory):
            for file in files:
                if file.endswith(('.nii', '.nii.gz')):
                    nifti_files.append(os.path.join(root, file))

        for file in nifti_files:
            nifti_image = nib.load(file)
            if len(nifti_image.shape) == 4:
                self.split4d(file, self.split_nifti_directory)
                nifti_files.remove(file)

        if len(nifti_files) == 1:
            print(" Error: Motion correction cannot be performed on a single 3D image.")

        return self.split_nifti_directory


def merge3d(nifti_dir: str, wild_card: str, nifti_outfile: str) -> None:
    """
    Merge 3D NIFTI files into a 4D NIFTI file using nibabel
    :param nifti_dir: Directory containing the 3D NIFTI files
    :param wild_card: Wildcard to use to find the 3D NIFTI files
    :param nifti_outfile: User-defined output file name for the 4D NIFTI file
    """
    logging.info(f"Merging 3D nifti files in {nifti_dir} with wildcard {wild_card}")
    files_to_merge = fop.get_files(nifti_dir, wild_card)
    nib.save(nib.funcs.concat_images(files_to_merge, False), nifti_outfile)
    os.chdir(nifti_dir)
    logging.info("Done")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
.. module:: input_validation
   :platform: Unix, Windows
   :synopsis: A module for input validation for the falconz.

.. moduleauthor:: Lalith Kumar Shiyam Sundar <lalith.shiyamsundar@meduniwien.ac.at>

This module handles input validation for the falconz.

Usage:
    The functions in this module can be imported and used in other modules within the falconz to perform image conversion.
"""
import logging
import os


class InputValidation:
    """
    A class for input validation for the falconz.

    :param args: The arguments to validate.
    :type args: Any
    """

    def __init__(self, args):
        self.args = args

    def validate(self):
        """
        Validates the input arguments.
        """
        self._check_directory_exists()
        self._check_reference_frame_index()
        self._check_start_frame()
        self._check_registration_type()
        self._check_multi_resolution_iterations()
        logging.info("Input validation successful.")

    def _check_directory_exists(self):
        """
        Checks if the specified directory exists.
        """
        if not os.path.exists(self.args.main_folder):
            raise ValueError(f"The specified directory does not exist: {self.args.main_folder}")

    def _check_reference_frame_index(self):
        """
        Checks if the reference frame index is valid.
        """
        if not isinstance(self.args.reference_frame_index, int) or self.args.reference_frame_index < -1:
            raise ValueError(
                f"Reference frame index must be a non-negative integer or -1: {self.args.reference_frame_index}")

    def _check_start_frame(self):
        """
        Checks if the start frame is valid.
        """
        if not isinstance(self.args.start_frame, int) or self.args.start_frame < 0:
            raise ValueError(f"Start frame must be a non-negative integer: {self.args.start_frame}")

    def _check_registration_type(self):
        """
        Checks if the registration type is valid.
        """
        allowed_registration_paradigms = ["rigid", "affine", "deformable"]
        if self.args.registration not in allowed_registration_paradigms:
            raise ValueError(f"Invalid registration type. Allowed values are: {allowed_registration_paradigms}")

    def _check_multi_resolution_iterations(self):
        """
        Checks if the multi-resolution iterations are valid.
        """
        if not isinstance(self.args.multi_resolution_iterations, str) or not all(
                i.isdigit() for i in self.args.multi_resolution_iterations.split('x')):
            raise ValueError(
                f"Multi resolution iterations must be a 'x' separated numeric string: {self.args.multi_resolution_iterations}")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
.. module:: falconz
   :platform: Unix, Windows
   :synopsis: Main module for the FalconZ project.

.. moduleauthor:: Lalith Kumar Shiyam Sundar <lalith.shiyamsundar@meduniwien.ac.at>

"""

# Importing necessary libraries and modules

import argparse
import colorama
import emoji
import logging
import multiprocessing
import os
import shutil
import sys
import time
from datetime import datetime

from falconz import constants
from falconz import display
from falconz import download
from falconz import file_utilities
from falconz import image_conversion
from falconz import input_validation
from falconz import resources
from falconz.constants import FALCON_WORKING_FOLDER
from falconz.image_conversion import ImageConverter, merge3d
from falconz.image_processing import determine_candidate_frames, align
from falconz.input_validation import InputValidation

logging.basicConfig(format='%(asctime)s %(levelname)-8s [%(filename)s:%(lineno)d] %(message)s', level=logging.INFO,
                    filename=datetime.now().strftime('falconz-v.1.0.0.%H-%M-%d-%m-%Y.log'),
                    filemode='w')


def main():
    """
    Main function to execute the FalconZ motion correction pipeline.
    
    It involves the following steps:
    
    1. Initialization (e.g. argument parsing)
    2. Input validation and preparation
    3. Downloading necessary binaries
    4. Standardizing input data
    5. Performing motion correction
    6. Cleaning up and finalizing results
    
    :param None: This function doesn't take any direct arguments, but processes command line arguments.

    :returns: None. But as a side-effect, produces motion-corrected images and other outputs.
    """
    colorama.init()

    # Initialization: Setting up arguments, parsers, etc.

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-m",
        "--main_folder",
        type=str,
        help="path containing the images to motion correct",
        required=True,
    )
    parser.add_argument(
        "-rf",
        "--reference_frame_index",
        type=int,
        default=-1,
        help="index of the reference frame [index starts from 0]",
    )
    parser.add_argument(
        "-sf",
        "--start_frame",
        type=int,
        default=99,
        help="frame from which the motion correction will be performed"
    )
    parser.add_argument(
        "-r",
        "--registration",
        type=str,
        choices=constants.ALLOWED_REGISTRATION_PARADIGMS,
        required=True,
        help="Type of registration: rigid  | affine  | deformable"
    )
    parser.add_argument(
        "-i",
        "--multi_resolution_iterations",
        type=str,
        default=constants.MULTI_RESOLUTION_SCHEME,
        help="Number of iterations for each resolution level"
    )
    args = parser.parse_args()
    validator = InputValidation(args)
    validator.validate()

    display.logo()
    display.citation()

    logging.info('----------------------------------------------------------------------------------------------------')
    logging.info('                                     STARTING FALCON-Z V.2.0.0                                      ')
    logging.info('----------------------------------------------------------------------------------------------------')

    # ----------------------------------
    # INPUT VALIDATION AND PREPARATION
    # ----------------------------------

    print(' ')
    print(f'{constants.ANSI_VIOLET} {emoji.emojize(":memo:")} NOTE:{constants.ANSI_RESET}')
    print(' ')
    display.expectations()
    display.default_parameters(args)
    display.derived_parameters(args)

    # ----------------------------------
    # DOWNLOADING THE BINARIES
    # ----------------------------------

    print('')
    print(f'{constants.ANSI_VIOLET} {emoji.emojize(":globe_with_meridians:")} BINARIES DOWNLOAD:{constants.ANSI_RESET}')

    print('')
    binary_path = constants.BINARY_PATH
    file_utilities.create_directory(binary_path)
    system_os, system_arch = file_utilities.get_system()
    print(f'{constants.ANSI_ORANGE} Detected system: {system_os} | Detected architecture: {system_arch}'
          f'{constants.ANSI_RESET}')
    download.download(item_name=f'greedy-{system_os}-{system_arch}', item_path=binary_path,
                      item_dict=resources.GREEDY_BINARIES)
    file_utilities.set_permissions(constants.GREEDY_PATH, system_os)
    download.download(item_name=f'c3d-{system_os}-{system_arch}', item_path=binary_path,
                      item_dict=resources.C3D_BINARIES)
    file_utilities.set_permissions(constants.C3D_PATH, system_os)

    # ----------------------------------
    # INPUT STANDARDIZATION
    # ----------------------------------

    print('')
    print(f'{constants.ANSI_VIOLET} {emoji.emojize(":magnifying_glass_tilted_left:")} STANDARDIZING INPUT DATA TO '
          f'NIFTI:{constants.ANSI_RESET}')
    print('')
    logging.info(' ')
    logging.info(' STANDARDIZING INPUT DATA TO NIFTI:')
    logging.info(' ')
    image_dir = args.main_folder
    parent_dir = os.path.dirname(image_dir)
    falcon_dir = os.path.join(parent_dir, FALCON_WORKING_FOLDER)
    file_utilities.create_directory(falcon_dir)
    image_converter = ImageConverter(input_directory=image_dir, output_directory=falcon_dir)
    split_nifti_dir = image_converter.convert()
    print(f"{constants.ANSI_GREEN} Standardization complete.{constants.ANSI_RESET}")
    logging.info(" Standardization complete.")

    # ----------------------------------
    # MOTION CORRECTION
    # ----------------------------------

    org_nifti_files = file_utilities.get_files(split_nifti_dir, '*.nii')
    if len(org_nifti_files) == 0:
        org_nifti_files = file_utilities.get_files(split_nifti_dir, '*.nii.gz')
    reference_file = org_nifti_files[args.reference_frame_index]
    candidate_frames = org_nifti_files.copy()
    candidate_frames.remove(reference_file)
    start_frame = args.start_frame
    # choose start frame automatically if not specified
    if args.start_frame == 99:
        n_jobs = multiprocessing.cpu_count()
        start_frame_file = determine_candidate_frames(candidate_frames, reference_file, falcon_dir,
                                                      round(n_jobs / 2))
        # find the index of the start_frame_file in the org_nifti_files list
        start_frame = org_nifti_files.index(start_frame_file)
    # everything from and after start_frame will be motion corrected and will be called moving frames
    moving_frames = org_nifti_files[start_frame:]
    moving_frames.remove(reference_file)
    # everything before that will not be motion corrected and will be called non-moco frames
    non_moco_frames = org_nifti_files[:start_frame]
    print('')
    print(f'{constants.ANSI_VIOLET} {emoji.emojize(":eagle:")} PERFORMING MOTION CORRECTION:{constants.ANSI_RESET}')
    print('')
    logging.info(' ')
    logging.info(' PERFORMING MOTION CORRECTION:')
    logging.info(' ')
    print(f' Number of files to motion correct: {len(candidate_frames)} | Reference file: '
          f'{os.path.basename(reference_file)} | Start frame: {start_frame}')
    moco_dir = os.path.join(falcon_dir, constants.MOCO_FOLDER)
    file_utilities.create_directory(moco_dir)
    align(fixed_img=reference_file, moving_imgs=moving_frames, registration_type=args.registration,
          multi_resolution_iterations=args.multi_resolution_iterations, moco_dir=moco_dir)

    # ----------------------------------
    # CLEANING UP
    # ----------------------------------

    # CLEAN TRANSFORMS
    transforms_dir = os.path.join(falcon_dir, constants.TRANSFORMS_FOLDER)
    file_utilities.create_directory(transforms_dir)
    for transform_keyword in constants.TRANSFORMS_KEYWORD:
        transform_files = file_utilities.get_files(split_nifti_dir, transform_keyword)
        if len(transform_files) > 0:
            for transform_file in transform_files:
                shutil.move(transform_file, transforms_dir)
        else:
            continue

    # COPY THE REFERENCE FRAME FROM THE SPLIT NIFTI FOLDER TO THE MOCO FOLDER WITH MOCO PREFIX
    reference_file_name = os.path.basename(reference_file)
    moco_reference_file = os.path.join(moco_dir, constants.MOCO_PREFIX + reference_file_name)
    shutil.copy(reference_file, moco_reference_file)

    # COPY THE NON-MOCO FRAMES FROM THE SPLIT NIFTI FOLDER TO THE MOCO FOLDER WITH MOCO PREFIX
    for non_moco_frame in non_moco_frames:
        non_moco_frame_name = os.path.basename(non_moco_frame)
        moco_non_moco_frame = os.path.join(moco_dir, constants.MOCO_PREFIX + non_moco_frame_name)
        shutil.copy(non_moco_frame, moco_non_moco_frame)

    # MERGE THE 3D MOCO FRAMES TO A 4D NIFTI FILE
    merge3d(moco_dir, constants.MOCO_PREFIX + '*', os.path.join(moco_dir, constants.MOCO_4D_FILE_NAME))
    print(f'{constants.ANSI_GREEN} Motion correction complete: '
          f'Results in {moco_dir} | 4D MoCo file: {constants.MOCO_4D_FILE_NAME}{constants.ANSI_RESET}')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
This module contains the constants that are used in the falconz.

The variables in this module can be imported and used in other modules within the falconz.
"""

import os
from datetime import datetime

from falconz import file_utilities

# Get the root directory of the virtual environment
project_root = file_utilities.get_virtual_env_root()

# Set the path to the binary folder
BINARY_PATH = os.path.join(project_root, 'bin')

# Set the paths to the binaries based on the operating system
if file_utilities.get_system()[0] == 'windows':
    GREEDY_PATH = os.path.join(BINARY_PATH, f'greedy-{file_utilities.get_system()[0]}-{file_utilities.get_system()[1]}',
                               'greedy.exe')
    C3D_PATH = os.path.join(BINARY_PATH, f'c3d-{file_utilities.get_system()[0]}-{file_utilities.get_system()[1]}',
                            'c3d.exe')
elif file_utilities.get_system()[0] in ['linux', 'mac']:
    GREEDY_PATH = os.path.join(BINARY_PATH, f'greedy-{file_utilities.get_system()[0]}-{file_utilities.get_system()[1]}',
                               'greedy')
    C3D_PATH = os.path.join(BINARY_PATH, f'c3d-{file_utilities.get_system()[0]}-{file_utilities.get_system()[1]}',
                            'c3d')
else:
    raise ValueError('Unsupported OS')

# Define color codes for console output
ANSI_ORANGE = '\033[38;5;208m'
ANSI_GREEN = '\033[38;5;40m'
ANSI_VIOLET = '\033[38;5;141m'
ANSI_RESET = '\033[0m'

# Define the allowed modalities
MODALITIES = ['PET', 'CT', 'MR']

# Define the minimum system requirements for each registration paradigm
MINIMUM_RAM_REQUIRED_RIGID = 4  # in GB
MINIMUM_RAM_REQUIRED_AFFINE = 8  # in GB
MINIMUM_RAM_REQUIRED_DEFORMABLE = 16  # in GB
MINIMUM_THREADS_REQUIRED_RIGID = 2  # in number of threads
MINIMUM_THREADS_REQUIRED_AFFINE = 4  # in number of threads
MINIMUM_THREADS_REQUIRED_DEFORMABLE = 8  # in number of threads

# Define the shrink levels supported
SHRINK_LEVEL = [2, 4, 8]

# Define the normalized cross correlation threshold and radius
NCC_THRESHOLD = 0.5
NCC_RADIUS = '4x4x4'

# Define the file names and folder names used in the FALCONZ pipeline
MOCO_PREFIX = 'moco_'
ALIGNED_PREFIX = 'aligned_'
TRANSFORMS_KEYWORD = ['*warp.nii.gz', '*rigid.mat', '*affine.mat']
MOCO_4D_FILE_NAME = 'moco_4D.nii.gz'
FALCON_WORKING_FOLDER = 'FALCONZ-V02' + '-' + datetime.now().strftime('%Y-%m-%d-%H-%M-%S')
TRANSFORMS_FOLDER = 'transforms'
MOCO_FOLDER = 'Motion-corrected-images'

# Define the hyperparameters used in the registration process
MULTI_RESOLUTION_SCHEME = '100x25x10'
EXPECTED_DIMENSIONS = 4
ALLOWED_REGISTRATION_PARADIGMS = ["rigid", "affine", "deformable"]
IMAGE_INTERPOLATION = 'Linear'
MASK_INTERPOLATION = 'Nearest Neighbor'
COST_FUNCTION = 'NCC 2x2x2'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
.. module:: resources
   :platform: Unix, Windows
   :synopsis: A module for storing urls and filenames of the binaries required for the falconz.

.. moduleauthor:: Lalith Kumar Shiyam Sundar <lalith.shiyamsundar@meduniwien.ac.at>

This module contains the urls and filenames of the binaries that are required for the falconz.

Usage:
    The variables in this module can be imported and used in other modules within the falconz to download the necessary
    binaries for the falconz.
"""

GREEDY_BINARIES = {
    "greedy-windows-x86_64": {
        "url": "https://greedy.s3.eu.cloud-object-storage.appdomain.cloud/greedy-windows-x86_64.zip",
        "filename": "greedy-windows-x86_64.zip",
        "directory": "greedy-windows-x86_64",
    },
    "greedy-linux-x86_64": {
        "url": "https://greedy.s3.eu.cloud-object-storage.appdomain.cloud/greedy-linux-x86_64.zip",
        "filename": "greedy-linux-x86_64.zip",
        "directory": "greedy-linux-x86_64",
    },
    "greedy-mac-x86_64": {
        "url": "https://greedy.s3.eu.cloud-object-storage.appdomain.cloud/greedy-mac-x86_64.zip",
        "filename": "greedy-mac-x86_64.zip",
        "directory": "greedy-mac-x86_64",
    },
    "greedy-mac-arm64": {
        "url": "https://greedy.s3.eu.cloud-object-storage.appdomain.cloud/greedy-mac-arm64.zip",
        "filename": "greedy-mac-arm64.zip",
        "directory": "greedy-mac-arm64",
    },
}

C3D_BINARIES = {
    "c3d-windows-x86_64": {
        "url": "https://greedy.s3.eu.cloud-object-storage.appdomain.cloud/c3d-windows-x86_64.zip",
        "filename": "c3d-windows-x86_64.zip",
        "directory": "c3d-windows-x86_64",
    },
    "c3d-linux-x86_64": {
        "url": "https://greedy.s3.eu.cloud-object-storage.appdomain.cloud/c3d-linux-x86_64.zip",
        "filename": "c3d-linux-x86_64.zip",
        "directory": "c3d-linux-x86_64",
    },
    "c3d-mac-x86_64": {
        "url": "https://greedy.s3.eu.cloud-object-storage.appdomain.cloud/c3d-mac-x86_64.zip",
        "filename": "c3d-mac-x86_64.zip",
        "directory": "c3d-mac-x86_64",
    },
    "c3d-mac-arm64": {
        "url": "https://greedy.s3.eu.cloud-object-storage.appdomain.cloud/c3d-mac-arm64.zip",
        "filename": "c3d-mac-arm64.zip",
        "directory": "c3d-mac-arm64",
    },
}
